@@ -7,10 +7,10 @@ import time
 import signal
 import socket
 import datetime
-import functools
 import io
 import re
 import random
+import subprocess
 import tempfile
 
 import portage
@@ -23,7 +23,6 @@ good = create_color_func("GOOD")
 bad = create_color_func("BAD")
 warn = create_color_func("WARN")
 from portage.const import VCS_DIRS, TIMESTAMP_FORMAT, RSYNC_PACKAGE_ATOM
-from portage.util._eventloop.global_event_loop import global_event_loop
 from portage.util import writemsg, writemsg_stdout
 from portage.util.futures import asyncio
 from portage.sync.getaddrinfo_validate import getaddrinfo_validate
@@ -60,13 +59,62 @@ class RsyncSync(NewBase):
 	def __init__(self):
 		NewBase.__init__(self, "rsync", RSYNC_PACKAGE_ATOM)
 
+	def _select_download_dir(self):
+		'''
+		Select and return the download directory. It's desirable to be able
+		to create shared hardlinks between the download directory to the
+		normal repository, and this is facilitated by making the download
+		directory be a subdirectory of the normal repository location
+		(ensuring that no mountpoints are crossed). Shared hardlinks are
+		created by using the rsync --link-dest option.
+
+		Since the download is initially unverified, it is safest to save
+		it in a quarantine directory. The quarantine directory is also
+		useful for making the repository update more atomic, so that it
+		less likely that normal repository location will be observed in
+		a partially synced state.
+
+		This method returns a quarantine directory if sync-allow-hardlinks
+		is enabled in repos.conf, and otherwise it returne the normal
+		repository location.
+		'''
+		if self.repo.sync_allow_hardlinks:
+			return os.path.join(self.repo.location, '.tmp-unverified-download-quarantine')
+		else:
+			return self.repo.location
+
+	def _commit_download(self, download_dir):
+		'''
+		Commit changes from download_dir if it does not refer to the
+		normal repository location.
+		'''
+		exitcode = 0
+		if self.repo.location != download_dir:
+			rsynccommand = [self.bin_command] + self.rsync_opts + self.extra_rsync_opts
+			rsynccommand.append('--exclude=/%s' % os.path.basename(download_dir))
+			rsynccommand.append('%s/' % download_dir.rstrip('/'))
+			rsynccommand.append('%s/' % self.repo.location)
+			exitcode = subprocess.call(rsynccommand)
+
+		return exitcode
+
+	def _remove_download(self, download_dir):
+		"""
+		Remove download_dir if it does not refer to the normal repository
+		location.
+		"""
+		exitcode = 0
+		if self.repo.location != download_dir:
+			exitcode = subprocess.call(['rm', '-rf', download_dir])
+		return exitcode
 
 	def update(self):
 		'''Internal update function which performs the transfer'''
 		opts = self.options.get('emerge_config').opts
 		self.usersync_uid = self.options.get('usersync_uid', None)
 		enter_invalid = '--ask-enter-invalid' in opts
-		out = portage.output.EOutput()
+		quiet = '--quiet' in opts
+		out = portage.output.EOutput(quiet=quiet)
 		syncuri = self.repo.sync_uri
 		if self.repo.module_specific_options.get(
 			'sync-rsync-vcs-ignore', 'false').lower() == 'true':
@@ -95,6 +143,9 @@ class RsyncSync(NewBase):
 			self.extra_rsync_opts.extend(portage.util.shlex_split(
 				self.repo.module_specific_options['sync-rsync-extra-opts']))
 
+		download_dir = self._select_download_dir()
+		exitcode = 0
+
 		# Process GLEP74 verification options.
 		# Default verification to 'no'; it's enabled for ::gentoo
 		# via default repos.conf though.
@@ -147,22 +198,11 @@ class RsyncSync(NewBase):
 			# will not be performed and the user will have to fix it and try again,
 			# so we may as well bail out before actual rsync happens.
 			if openpgp_env is not None and self.repo.sync_openpgp_key_path is not None:
-
 				try:
 					out.einfo('Using keys from %s' % (self.repo.sync_openpgp_key_path,))
 					with io.open(self.repo.sync_openpgp_key_path, 'rb') as f:
 						openpgp_env.import_key(f)
-					out.ebegin('Refreshing keys from keyserver')
-					retry_decorator = self._key_refresh_retry_decorator()
-					if retry_decorator is None:
-						openpgp_env.refresh_keys()
-					else:
-						loop = global_event_loop()
-						func_coroutine = functools.partial(loop.run_in_executor,
-							None, openpgp_env.refresh_keys)
-						decorated_func = retry_decorator(func_coroutine)
-						loop.run_until_complete(decorated_func())
-					out.eend(0)
+					self._refresh_keys(openpgp_env)
 				except (GematoException, asyncio.TimeoutError) as e:
 					writemsg_level("!!! Manifest verification impossible due to keyring problem:\n%s\n"
 							% (e,),
@@ -200,8 +240,10 @@ class RsyncSync(NewBase):
 				self.proto = "file"
 				dosyncuri = syncuri[7:]
 				unchanged, is_synced, exitcode, updatecache_flg = self._do_rsync(
-					dosyncuri, timestamp, opts)
+					dosyncuri, timestamp, opts, download_dir)
 				self._process_exitcode(exitcode, dosyncuri, out, 1)
+				if exitcode == 0 and not unchanged:
+					self._commit_download(download_dir)
 				return (exitcode, updatecache_flg)
 
 			retries=0
@@ -333,7 +375,7 @@ class RsyncSync(NewBase):
 					dosyncuri = dosyncuri[6:].replace('/', ':/', 1)
 
 				unchanged, is_synced, exitcode, updatecache_flg = self._do_rsync(
-					dosyncuri, timestamp, opts)
+					dosyncuri, timestamp, opts, download_dir)
 				if not unchanged:
 					local_state_unchanged = False
 				if is_synced:
@@ -350,6 +392,12 @@ class RsyncSync(NewBase):
 					break
 			self._process_exitcode(exitcode, dosyncuri, out, maxretries)
 
+			if local_state_unchanged:
+				# The quarantine download_dir is not intended to exist
+				# in this case, so refer gemato to the normal repository
+				# location.
+				download_dir = self.repo.location
+
 			# if synced successfully, verify now
 			if exitcode == 0 and self.verify_metamanifest:
 				if gemato is None:
@@ -361,7 +409,7 @@ class RsyncSync(NewBase):
 						# we always verify the Manifest signature, in case
 						# we had to deal with key revocation case
 						m = gemato.recursiveloader.ManifestRecursiveLoader(
-								os.path.join(self.repo.location, 'Manifest'),
+								os.path.join(download_dir, 'Manifest'),
 								verify_openpgp=True,
 								openpgp_env=openpgp_env,
 								max_jobs=self.verify_jobs)
@@ -373,10 +421,12 @@ class RsyncSync(NewBase):
 							raise RuntimeError('Timestamp not found in Manifest')
 						if (self.max_age != 0 and
 								(datetime.datetime.utcnow() - ts.ts).days > self.max_age):
+							out.quiet = False
 							out.ewarn('Manifest is over %d days old, this is suspicious!' % (self.max_age,))
 							out.ewarn('You may want to try using another mirror and/or reporting this one:')
 							out.ewarn('  %s' % (dosyncuri,))
 							out.ewarn('')
+							out.quiet = quiet
 
 						out.einfo('Manifest timestamp: %s UTC' % (ts.ts,))
 						out.einfo('Valid OpenPGP signature found:')
@@ -390,7 +440,7 @@ class RsyncSync(NewBase):
 						# if nothing has changed, skip the actual Manifest
 						# verification
 						if not local_state_unchanged:
-							out.ebegin('Verifying %s' % (self.repo.location,))
+							out.ebegin('Verifying %s' % (download_dir,))
 							m.assert_directory_verifies()
 							out.eend(0)
 					except GematoException as e:
@@ -399,12 +449,16 @@ class RsyncSync(NewBase):
 								level=logging.ERROR, noiselevel=-1)
 						exitcode = 1
 
+			if exitcode == 0 and not local_state_unchanged:
+				exitcode = self._commit_download(download_dir)
+
 			return (exitcode, updatecache_flg)
 		finally:
+			if exitcode == 0:
+				self._remove_download(download_dir)
 			if openpgp_env is not None:
 				openpgp_env.close()
 
-
 	def _process_exitcode(self, exitcode, syncuri, out, maxretries):
 		if (exitcode==0):
 			pass
@@ -540,7 +594,7 @@ class RsyncSync(NewBase):
 		return rsync_opts
 
 
-	def _do_rsync(self, syncuri, timestamp, opts):
+	def _do_rsync(self, syncuri, timestamp, opts, download_dir):
 		updatecache_flg = False
 		is_synced = False
 		if timestamp != 0 and "--quiet" not in opts:
@@ -665,6 +719,12 @@ class RsyncSync(NewBase):
 			elif (servertimestamp == 0) or (servertimestamp > timestamp):
 				# actual sync
 				command = rsynccommand[:]
+
+				if self.repo.location != download_dir:
+					# Use shared hardlinks for files that are identical
+					# in the previous snapshot of the repository.
+					command.append('--link-dest=%s' % self.repo.location)
+
 				submodule_paths = self._get_submodule_paths()
 				if submodule_paths:
 					# The only way to select multiple directories to
@@ -675,9 +735,10 @@ class RsyncSync(NewBase):
 						# /./ is special syntax supported with the
 						# rsync --relative option.
 						command.append(syncuri + "/./" + path)
-					command.append(self.repo.location)
 				else:
-					command.extend([syncuri + "/", self.repo.location])
+					command.append(syncuri + "/")
+
+				command.append(download_dir)
 
 				exitcode = None
 				try:
